// Copyright 2023 KUKA Deutschland GmbH
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef OS_CORE_COMM_UDP_PUBLISHER_H
#define OS_CORE_COMM_UDP_PUBLISHER_H

#include "socket.h"

namespace os::core::udp::communication {

class Publisher {
 public:
  typedef Socket::ErrorCode ErrorCode;

 public:
  Publisher(const SocketAddress& pub_address, const SocketAddress& interface_address,
            bool is_multicast);
  virtual ~Publisher() = default;
  ErrorCode Setup();

  ErrorCode Send(const uint8_t* msg_data, uint16_t msg_size);
  bool SetTTL(int ttl);

 public:
  const SocketAddress& Address() const { return pub_address_; }
  bool IsMulticast() const { return is_multicast_; }

 protected:
  Socket socket_;
  SocketAddress pub_address_;
  SocketAddress interface_address_;
  const bool is_multicast_;
};
}  // namespace os::core::udp::communication

#endif  // OS_CORE_COMM_UDP_PUBLISHER_H
